const opsSchema = require('./schemas/ops');
const validator = require('./validator');

const validate = validator(opsSchema);

function execHandler(method) {
  function handle(params) {
    const ctx = {
      swatchCtx: {},
    };
    method.validate(ctx, params);
    return method.handle(ctx);
  }

  return handle;
}

function batch(model) {
  const methods = {};
  model.forEach((method) => {
    methods[method.name] = execHandler(method);
  });

  function handleOp(op) {
    if (op.method in methods) {
      try {
        const result = methods[op.method](op.args);
        return {
          ok: true,
          result,
        };
      } catch (e) {
        return {
          ok: false,
          error: e.message,
        };
      }
    } else {
      return {
        ok: false,
        error: 'invalid_method',
      };
    }
  }

  function batchHandler(ops) {
    try {
      validate(ops);
    } catch (e) {
      throw new Error('invalid_request');
    }

    return ops.map(handleOp);
  }

  return batchHandler;
}

module.exports = batch;
